@@ -77,7 +77,7 @@ pmomPM <- function(y, x, xadj, niter=10^4, thinning=1, burnin=round(niter/10), p
     ndeltaini <- as.integer(sum(msfit)); deltaini <- as.integer(msfit)
     if (verbose) cat(" Done\n")
   } else if (initSearch=='SCAD') {
-    require(ncvreg)
+    #require(ncvreg)
     if (verbose) cat("Initializing via SCAD cross-validation...")
     deltaini <- rep(TRUE,ncol(x))
     cvscad <- cv.ncvreg(X=x,y=y-mean(y),family="gaussian",penalty="SCAD",nfolds=10,dfmax=1000,max.iter=10^4)
@@ -133,7 +133,7 @@ pmomPMR <- function(y, x, xadj, r=1, tau, tau.adj=10^6, a.tau=1, b.tau=.135, nit
 # Output: list with 2 elements
 # - postSample: posterior samples
 # - margpp: marginal posterior probability for inclusion of each covariate (approx by averaging marginal post prob for inclusion in each Gibbs iteration. This approx is more accurate than simply taking colMeans(postSample)).
-require(mvtnorm)
+#require(mvtnorm)
 if (missing(tau)) { unknownTau <- TRUE } else { unknownTau <- FALSE }
 if (is.character(y)) { y <- as.numeric(factor(y))-1 } else if (is.factor(y)) { y <- as.numeric(y)-1 }
 if (length(unique(y))>2) stop('y has more than 2 levels')
@@ -155,7 +155,7 @@ if (initSearch=='none') {
   postTheta1[1,] <- rep(0,p1)
 } else if (initSearch=='SCAD') {
   if (verbose) cat("Initializing via SCAD cross-validation")
-  require(ncvreg)
+  #require(ncvreg)
   warn <- options('warn')$warn; options(warn= -1)
   cvscad <- cv.ncvreg(X=x,y=y,family="binomial",penalty="SCAD",nfolds=10,dfmax=1000,max.iter=10^4)
   postTheta1[1,] <- ncvreg(X=x,y=y,penalty="SCAD",dfmax=1000,lambda=rep(cvscad$lambda[cvscad$cv],2))$beta[-1, 1]
