@@ -2,7 +2,7 @@
 ## Routines to simulate from MOM prior and posterior
 ##################################################################################
 
-setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='msfit'), function(y, x, m, V,msfit, priorCoef, priorVar, niter=10^3, burnin=round(niter/10), thinning=1) {
+setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='msfit'), function(y, x, m, V,msfit, priorCoef, priorVar=igprior(alpha=0.01,lambda=0.01), niter=10^3, burnin=round(niter/10), thinning=1) {
   if (!(class(y) %in% c('numeric','Surv'))) stop("y must be of class 'numeric' or 'Surv'")
   #Draw model
   pp <- postProb(msfit)
@@ -35,30 +35,46 @@ setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='ms
 )
 
 
-setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='missing'), function(y, x, m, V, msfit, priorCoef, priorVar, niter=10^3, burnin=round(niter/10), thinning=1) {
+setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='missing'), function(y, x, m, V, msfit, priorCoef, priorVar=igprior(alpha=0.01,lambda=0.01), niter=10^3, burnin=round(niter/10), thinning=1) {
+  tau <- as.double(priorCoef@priorPars['tau'])
   if (class(y) == 'numeric') {  ##Linear model
     p <- ncol(x); n <- length(y)
     if (nrow(x) != n) stop('Dimensions of y and x do not match')
-    tau <- as.double(priorCoef@priorPars['tau'])
-    if (priorCoef@priorDistr=='pMOM') {
-      prior <- as.integer(0); r <- as.integer(priorCoef@priorPars['r'])
-    } else if (priorCoef@priorDistr=='piMOM') {
-      prior <- as.integer(1); r <- as.integer(0)
-    } else if (priorCoef@priorDistr=='peMOM') {
-      prior <- as.integer(2); r <- as.integer(0)
-    } else stop("This kind of prior is not implemented")
     if (priorVar@priorDistr=='invgamma') {
-      a_phi <- as.double(priorVar@priorPars['alpha'])
-      b_phi <- as.double(priorVar@priorPars['lambda'])
+        a_phi <- as.double(priorVar@priorPars['alpha'])
+        b_phi <- as.double(priorVar@priorPars['lambda'])
     } else stop("Only invgamma prior for residual variance is currently implemented")
-    if (p==0) {
-      ans <- matrix(1/rgamma((niter-burnin)/thinning, .5*(a_phi+n), .5*(b_phi+sum(y^2))), ncol=1)
-      colnames(ans) <- 'phi'
-    } else {
-      ans <- .Call("rnlpPostCI_lm",as.integer(niter),as.integer(burnin),as.integer(thinning),as.double(y),as.double(x),as.integer(p),as.integer(r),tau,a_phi,b_phi,prior)
-      ans <- matrix(ans,ncol=p+1)
-      if (is.null(colnames(x))) colnames(ans) <- c(paste('beta',1:ncol(x),sep=''),'phi') else colnames(ans) <- c(colnames(x),'phi')
-    }
+    if (priorCoef@priorDistr %in% c('pMOM','peMOM','piMOM')) {
+      if (priorCoef@priorDistr=='pMOM') {
+        prior <- as.integer(0); r <- as.integer(priorCoef@priorPars['r'])
+      } else if (priorCoef@priorDistr=='piMOM') {
+        prior <- as.integer(1); r <- as.integer(0)
+      } else {
+        prior <- as.integer(2); r <- as.integer(0)
+      }
+      if (p==0) {
+        ans <- matrix(1/rgamma((niter-burnin)/thinning, .5*(a_phi+n), .5*(b_phi+sum(y^2))), ncol=1)
+        colnames(ans) <- 'phi'
+      } else {
+        ans <- .Call("rnlpPostCI_lm",as.integer(niter),as.integer(burnin),as.integer(thinning),as.double(y),as.double(x),as.integer(p),as.integer(r),tau,a_phi,b_phi,prior)
+        ans <- matrix(ans,ncol=p+1)
+        if (is.null(colnames(x))) colnames(ans) <- c(paste('beta',1:ncol(x),sep=''),'phi') else colnames(ans) <- c(colnames(x),'phi')
+      }
+    } else if (priorCoef@priorDistr=='zellner') {
+      if (p==0) {
+        ans <- matrix(1/rgamma((niter-burnin)/thinning, .5*(a_phi+n), .5*(b_phi+sum(y^2))), ncol=1)
+        colnames(ans) <- 'phi'
+      } else {
+        S <- solve((1+1/tau) * t(x) %*% x)
+        m <- as.vector(S %*% t(x) %*% matrix(y,ncol=1))
+        ssr <- sum(y * (y - (x %*% m)))
+        phi <- 1 / rgamma((niter - burnin)/thinning, 0.5*(n+a_phi), 0.5*(ssr + b_phi))
+        beta <- matrix(rnorm(ncol(x)*(niter-burnin)/thinning),ncol=ncol(x)) %*% t(chol(S)) * phi
+        beta <- t(t(beta)+m)
+        ans <- cbind(beta, phi)
+        if (is.null(colnames(x))) colnames(ans) <- c(paste('beta',1:ncol(x),sep=''),'phi') else colnames(ans) <- c(colnames(x),'phi')
+      }
+    } else stop("This kind of prior is not implemented")
   } else if (class(y)=='Surv') {  #Cox model
     p <- ncol(x)
     if (p==0) {
@@ -66,7 +82,12 @@ setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='mi
     } else {
       fit <- coxph(y ~ ., data=data.frame(x))
       thhat <- matrix(coef(fit),ncol=1); Vinv <- solve(fit$var)
-      Sinv <- Vinv + diag(p)/tau; S <- solve(Sinv)
+      if (priorCoef@priorDistr %in% c('pMOM','peMOM','piMOM')) {
+        Sinv <- Vinv + diag(p)/tau
+      } else if (priorCoef@priorDistr=='zellner') {
+        Sinv <- Vinv * (1+1/tau)
+      } else stop("This kind of prior is not implemented")
+      S <- solve(Sinv)
       m <- S %*% Vinv %*% thhat
       ans <- rnlp(m=as.vector(m), V=S, priorCoef=priorCoef, niter=niter, burnin=burnin, thinning=thinning)
       if (is.null(colnames(x))) colnames(ans) <- paste('beta',1:ncol(x),sep='') else colnames(ans) <- colnames(x)
@@ -79,18 +100,22 @@ setMethod("rnlp", signature(y='ANY',x='matrix',m='missing',V='missing',msfit='mi
 )
 
 
-setMethod("rnlp", signature(y='missing',x='missing',m='numeric',V='matrix',msfit='missing'), function(y, x, m, V, msfit, priorCoef, priorVar, niter=10^3, burnin=round(niter/10), thinning=1) {
+setMethod("rnlp", signature(y='missing',x='missing',m='numeric',V='matrix',msfit='missing'), function(y, x, m, V, msfit, priorCoef, priorVar=igprior(alpha=0.01,lambda=0.01), niter=10^3, burnin=round(niter/10), thinning=1) {
   p <- ncol(V)
   tau <- as.double(priorCoef@priorPars['tau'])
-  if (priorCoef@priorDistr=='pMOM') {
-    prior <- as.integer(0); r <- as.integer(priorCoef@priorPars['r'])
-  } else if (priorCoef@priorDistr=='piMOM') {
-    prior <- as.integer(1); r <- as.integer(0)
-  } else if (priorCoef@priorDistr=='peMOM') {
-    prior <- as.integer(2); r <- as.integer(0)
+  if (priorCoef@priorDistr %in% c('pMOM','peMOM','piMOM')) {
+    if (priorCoef@priorDistr=='pMOM') {
+      prior <- as.integer(0); r <- as.integer(priorCoef@priorPars['r'])
+    } else if (priorCoef@priorDistr=='piMOM') {
+      prior <- as.integer(1); r <- as.integer(0)
+    } else if (priorCoef@priorDistr=='peMOM') {
+      prior <- as.integer(2); r <- as.integer(0)
+    } else stop("This kind of prior is not implemented")
+    ans <- .Call("rnlpCI",as.integer(niter),as.integer(burnin),as.integer(thinning),as.double(m),as.double(V),as.integer(p),as.integer(r),tau,prior)
+    ans <- matrix(ans,ncol=p)
+  } else if (priorCoef@priorDistr == 'zellner') {
+    ans <- matrix(rnorm(p*(niter-burnin)/thinning),nrow=p) %*% t(chol(V))
   } else stop("This kind of prior is not implemented")
-  ans <- .Call("rnlpCI",as.integer(niter),as.integer(burnin),as.integer(thinning),as.double(m),as.double(V),as.integer(p),as.integer(r),tau,prior)
-  ans <- matrix(ans,ncol=p)
   if (is.null(names(m))) colnames(ans) <- paste('beta',1:length(m),sep='') else colnames(ans) <- names(m)
   return(ans)
 }
